//go:build wireinject
// +build wireinject

package cmd

import (
	"leafdev.top/Leaf/leaf-library-3/internal/api"
	"leafdev.top/Leaf/leaf-library-3/internal/base"
	"leafdev.top/Leaf/leaf-library-3/internal/base/conf"
	"leafdev.top/Leaf/leaf-library-3/internal/base/logger"
	"leafdev.top/Leaf/leaf-library-3/internal/base/milvus"
	"leafdev.top/Leaf/leaf-library-3/internal/base/orm"
	"leafdev.top/Leaf/leaf-library-3/internal/base/redis"
	"leafdev.top/Leaf/leaf-library-3/internal/base/s3"
	"leafdev.top/Leaf/leaf-library-3/internal/base/server"
	"leafdev.top/Leaf/leaf-library-3/internal/batch"
	"leafdev.top/Leaf/leaf-library-3/internal/dao"
	"leafdev.top/Leaf/leaf-library-3/internal/router"
	"leafdev.top/Leaf/leaf-library-3/internal/service"

	"github.com/google/wire"
)

var ProviderSet = wire.NewSet(
	conf.NewConfig,
	logger.NewZapLogger,
	orm.NewGORM,
	dao.NewQuery,
	redis.NewRedis,
	s3.NewS3,
	milvus.NewService,
	batch.NewBatch,
	service.Provide,
	api.Provide,
	router.Provide,
	server.NewHTTPServer,
	base.NewApplication,
)

func CreateApp() (*base.Application, error) {
	wire.Build(ProviderSet)

	return nil, nil
}
